/** \file fifo.h
 *
 * (c) 2009-2011 Martin Strubel <hackfin@section5.ch>
 *
 * See LICENSE.txt in this distribution for usage terms.
 *
 */

#define TO_SIM   0  ///< FIFO C -> Simulator
#define FROM_SIM 1  ///< FIFO Simulator -> C
#define OVR      2  ///< Overrun bit
#define UNR      3  ///< Underrun bit

/** The software FIFO descriptor structure */
struct fifo_t {
	unsigned char *buf;
	unsigned short size;
	unsigned short head;
	unsigned short tail;
	unsigned short fill;
	unsigned char  ovr;   // Overrun (wrote when full)
	unsigned char  unr;   // Underrun (read when empty)
};

typedef struct fifo_t Fifo;

/** Initialize a FIFO with given size */

int fifo_init(Fifo *f, unsigned short size);

/** Release FIFO resources */

void fifo_exit(Fifo *f);

/** Read from FIFO
 * \param buf    Buffer to copy to from FIFO
 * \param n      Number of bytes to read
 *
 * \return       Number of bytes read
 *
 */

int fifo_read(Fifo *f, unsigned char *buf, unsigned short n);

/** Write to FIFO
 * \param buf    Buffer to copy from to FIFO
 * \param n      Number of bytes to read
 *
 * \return       Number of bytes read
 *
 */

int fifo_write(Fifo *f, const unsigned char *buf, unsigned short n);

/** Returns FIFO status depending on the 'which' argument:
 * #FIFO_WRITE: Check if bytes can be written
 * #FIFO_READ:  Check if bytes available from FIFO
 *
 * \param which [#FIFO_WRITE, #FIFO_READ]
 * \return      1: FIFO ready, 0: Full/empty
 */

int fifo_status(Fifo *f, char which);

#define FIFO_WRITE 0 ///< Select FIFO_WRITE queue
#define FIFO_READ  1 ///< Select FIFO_READ queue

/** Return FIFO fill state (number of bytes) */
int fifo_fill(Fifo *f);

